/*
 * Copyright 2014-2015 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.storage;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.testbase.TestBaseQuery;
import ru.ispras.testbase.TestBaseQueryResult;
import ru.ispras.testbase.TestData;
import ru.ispras.testbase.knowledge.iterator.Iterator;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class TestBase {
  private final TestBaseStorage storage;
  private final Map<String, DataConverter> converters;

  public TestBase(final TestBaseStorage storage) {
    InvariantChecks.checkNotNull(storage);

    this.storage = storage;
    this.converters = new HashMap<>();
  }

  public DataConverter addDataConverter(final DataConverter converter) {
    InvariantChecks.checkNotNull(converter);

    return converters.put(converter.getName(), converter);
  }

  public TestBaseQueryResult executeQuery(final TestBaseQuery query) {
    InvariantChecks.checkNotNull(query);

    final StoredTestData data = storage.fetch(query);
    final DataConverter converter = converters.get(data.getConverterName());

    if (converters == null) {
      return TestBaseQueryResult.reportErrors(
          Collections.singletonList("Missing required converter"));
    }

    final Iterator<TestData> dataIterator = converter.convert(data, query);
    return TestBaseQueryResult.success(dataIterator);
  }
}
